#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <openobex/obex.h>

#define HEADER_CREATOR_ID 0xcf
#define HEADER_DESCRIPTION 0x05

obex_t *handle = NULL;
volatile int finished = 0;

void empeg_command(const uint8_t *cmd) {
	int notify_fd;
	char *serial_cmd;

	if((notify_fd = open("/proc/empeg_notify", O_WRONLY)) != -1) {
		serial_cmd = malloc(strlen(cmd) + 9);
		snprintf(serial_cmd, strlen(cmd) + 9, "SERIAL=%s\n", cmd);
		printf("Empire: Performing Serial command: %s", serial_cmd);
		write(notify_fd, serial_cmd, strlen(cmd) + 9);
		free(serial_cmd);
		close(notify_fd);
	}
}

void put_done(obex_object_t *object) {

	obex_headerdata_t header_data;
	uint8_t header_ident;
	int i, hlen, body_len = 0;
	const uint8_t *uint8_t_body = NULL;
	char *name = NULL, *namebuf = NULL;
	char fid[11];

	while(OBEX_ObjectGetNextHeader(handle, object, &header_ident, &header_data, &hlen)) {
		switch(header_ident) {

		case OBEX_HDR_BODY:
			uint8_t_body = header_data.bs;
			body_len = hlen;
			break;

		case OBEX_HDR_NAME:
			if((namebuf = malloc(hlen / 4))) {
				OBEX_UnicodeToChar(namebuf, header_data.bs, hlen);
				name = namebuf;
			}
			break;

		case HEADER_DESCRIPTION:	//recieved a part of the description
		case OBEX_HDR_LENGTH:		//recieved the length of the header
		case HEADER_CREATOR_ID:		//recieved the creator id
		default:			//other header we don't care about
		}
	}
	
	free(namebuf);

	//if(strcmp(name, "control.empeg") == 0) empeg_command(body);
	if(strcmp(name, "palantir_insert.empeg") == 0) {
		//printf("Name: %s\n", name);	

		for(i = body_len - 8; i >= 0; i -= 8) {
			strcpy(fid, "#XXXXXXXX0!");
			memcpy(fid + 1, (uint8_t_body + i), 8);
			//printf("%s\n", fid);
			empeg_command(fid);
		}
	}
}

void server_request(obex_object_t *object, int event, int cmd) {
	switch(cmd) {
	case OBEX_CMD_SETPATH:
		OBEX_ObjectSetRsp(object, OBEX_RSP_CONTINUE, OBEX_RSP_SUCCESS);
		break;
	case OBEX_CMD_PUT:
		OBEX_ObjectSetRsp(object, OBEX_RSP_CONTINUE, OBEX_RSP_SUCCESS);
		put_done(object);
		break;
	case OBEX_CMD_CONNECT:
	case OBEX_CMD_DISCONNECT:
		OBEX_ObjectSetRsp(object, OBEX_RSP_SUCCESS, OBEX_RSP_SUCCESS);
		break;
	default:
		OBEX_ObjectSetRsp(object, OBEX_RSP_NOT_IMPLEMENTED, OBEX_RSP_NOT_IMPLEMENTED);
		break;
	}
	return;
}

void obex_event_handler(obex_t *handle, obex_object_t *object, int mode, int event, int obex_cmd, int obex_rsp) {
	switch (event)	{
	case OBEX_EV_PROGRESS:	// a piece has been recieved
		break;
	case OBEX_EV_REQDONE:
		if(obex_cmd == OBEX_CMD_DISCONNECT) {
			finished = 1;
			OBEX_TransportDisconnect(handle);
		}
		break;
	case OBEX_EV_REQHINT:
		switch(obex_cmd) {
		case OBEX_CMD_PUT:		//Getting object
		case OBEX_CMD_CONNECT:
		case OBEX_CMD_DISCONNECT:
			OBEX_ObjectSetRsp(object, OBEX_RSP_CONTINUE, OBEX_RSP_SUCCESS);
			break;
		default:
			OBEX_ObjectSetRsp(object, OBEX_RSP_NOT_IMPLEMENTED, OBEX_RSP_NOT_IMPLEMENTED);
			break;
		}
		break;
	case OBEX_EV_REQ:
		// Comes when a server-request has been completed
		server_request(object, event, obex_cmd);
		break;
	case OBEX_EV_LINKERR:
		finished = 1;
		break;
	default:
	}
}

int main(void) {
	//pid_t pid;
	//if((pid = fork()) < 0)
	//	exit(EXIT_FAILURE);
	//if(pid > 0)
	//	exit(EXIT_SUCCESS);
	daemon(0, 0);
	
	handle = OBEX_Init(OBEX_TRANS_IRDA, obex_event_handler, 0);
	printf("Empire: Server waiting for OBEX objects.\n");

	while(1) {
		IrOBEX_ServerRegister(handle, "OBEX");
		while (!finished) OBEX_HandleInput(handle, 1);
		finished = 0;
	}
	return 0;
}


